# Threads in Java

This course material introduces the concept of threads in Java, covering their creation, management, and common challenges.  We'll explore how threads enable concurrent execution, enhancing the performance and responsiveness of applications.

## 1. Introduction to Threads

A thread is a lightweight unit of execution within a process.  It allows a program to perform multiple tasks seemingly at the same time.  Java provides robust support for multithreading through the `Thread` class and the `Runnable` interface.

**Why use threads?**

* **Improved Responsiveness:**  Applications can remain responsive to user input even while performing lengthy operations.
* **Resource Sharing:** Threads share the same memory space, facilitating efficient communication and data exchange.
* **Simplified Program Structure:** Concurrent tasks can be represented as separate threads, improving code organization.
* **Enhanced Performance:** On multi-core processors, threads can run truly concurrently, maximizing resource utilization.

## 2. Creating and Starting Threads

There are two primary ways to create threads in Java:

**2.1. Extending the `Thread` class:**

```java
class MyThread extends Thread {
    @Override
    public void run() {
        // Code to be executed by the thread
        System.out.println("Running from MyThread");
    }
}

public class Main {
    public static void main(String[] args) {
        MyThread thread = new MyThread();
        thread.start(); // Start the thread
    }
}
```

**2.2. Implementing the `Runnable` interface:**

```java
class MyRunnable implements Runnable {
    @Override
    public void run() {
        // Code to be executed by the thread
        System.out.println("Running from MyRunnable");
    }
}

public class Main {
    public static void main(String[] args) {
        MyRunnable runnable = new MyRunnable();
        Thread thread = new Thread(runnable);
        thread.start(); // Start the thread
    }
}
```

The `start()` method creates a new operating system thread and invokes the `run()` method.  It is crucial to use `start()` and not call `run()` directly, as the latter would execute the code within the main thread.

## 3. Thread Life Cycle

A thread progresses through several states during its lifetime:

* **New:**  The thread is created but not yet started.
* **Runnable:** The thread is ready to run and is either running or waiting for a processor.
* **Blocked:** The thread is waiting for a monitor lock.
* **Waiting:** The thread is waiting for a specific condition.
* **Timed Waiting:** The thread is waiting for a specified duration.
* **Terminated:** The thread has completed execution.


## 4. Thread Synchronization and Concurrency Control

When multiple threads access shared resources, synchronization mechanisms are essential to prevent data corruption and ensure predictable behavior.

**4.1. `synchronized` keyword:**

The `synchronized` keyword provides mutual exclusion, preventing multiple threads from accessing a critical section simultaneously.

```java
public class Counter {
    private int count = 0;

    public synchronized void increment() {
        count++;
    }
}
```

**4.2. Locks (e.g., `ReentrantLock`):**

`ReentrantLock` offers more flexibility than `synchronized`, including fairness policies and interruptible locks.

```java
import java.util.concurrent.locks.ReentrantLock;

public class Counter {
    private int count = 0;
    private ReentrantLock lock = new ReentrantLock();

    public void increment() {
        lock.lock();
        try {
            count++;
        } finally {
            lock.unlock();
        }
    }
}
```


## 5. Deadlocks

A deadlock occurs when two or more threads are blocked indefinitely, waiting for each other to release resources. Careful design and resource management are crucial to avoid deadlocks.


## 6. Further Reading

* **Java Concurrency in Practice by Brian Goetz et al.:** A comprehensive guide to concurrent programming in Java.  ([Find online](https://www.amazon.com/Java-Concurrency-Practice-Brian-Goetz/dp/0321349601))
* **Oracle's Java Tutorials: Concurrency:** Official documentation providing detailed information about Java threads and concurrency. ([Link to Documentation](https://docs.oracle.com/javase/tutorial/essential/concurrency/))
* **Article: "Understanding Java Threads" by Jakob Jenkov:** ([Example Article](http://tutorials.jenkov.com/java-concurrency/creating-and-starting-threads.html)) (Note:  Replace with a real, relevant article)



This course material provides a foundational understanding of threads in Java.  Remember to practice and explore further to gain deeper insights into the nuances of concurrent programming.  Effective use of threads can significantly enhance the performance and responsiveness of your Java applications.
